/*
	Renders a UI vertex given the screen to viewport matrix
*/

precision highp float;

uniform mat4 WorldToView;
uniform vec3 CameraUp;
uniform vec3 CameraRight;

attribute vec3 Position;
attribute vec3 Normal;
attribute vec4 Color;
attribute vec2 Tex0;

varying lowp vec4 Out_Color;
varying mediump vec2 Out_Tex0;

vec2 Rotate2D(vec2 xy, float rotation) {
	return vec2(xy.x * cos(rotation) - xy.y * sin(rotation),
				  xy.x * sin(rotation) + xy.y * cos(rotation));
}

void main()
{
    // compute world space corners of the particle quad accounting for rotation
    vec2 offsetMult = vec2(-2.0 * Tex0.x + 1.0, -2.0 * Tex0.y + 1.0) * Normal.xy;
    // offsetMult = Rotate2D(Normal.xy * offsetMult, Normal.z);		// Normal z holds the rotation
    vec3 WorldPos = Position + offsetMult.x*vec3(1,0,0) + offsetMult.y*vec3(0,1,0);
	
	// move to view space
    gl_Position = vec4(WorldPos,1) * WorldToView;
	
	// pass through the color and texture coordinate
    Out_Color = Color;
    Out_Tex0 = Tex0;
}