/*
	Transforms a normal'ed rigid mesh into world space with lighting calculations
*/

precision highp float;

uniform mat4 WorldToView;
uniform mat4 ObjToWorld;
uniform mat4 LightViewProjection;
uniform vec3 EyePosition;
uniform mediump vec4 DirFromLight;
uniform lowp vec4 LightColor;
uniform lowp vec4 AmbientColor;
 
attribute vec3 Position;
attribute vec3 Normal;
attribute vec2 Tex0;

varying mediump vec2 Out_Tex0;
varying mediump vec3 Out_Normal;
varying mediump vec3 Out_DirEye;
varying mediump vec2 Out_LightTex0;
varying lowp vec4 Out_Lighting;

uniform vec4 RimParam;
uniform vec4 SpecParam;

float GetSpecularity( vec3 eyeDir, vec3 lightDir, vec3 normal, float specPower, float specMult)
{    
    vec3 reflLight = lightDir - 2.0*normal*dot(lightDir,normal);
    float rDotV = clamp(dot(reflLight, eyeDir), 0.000001, 1.0);
	float specularity = pow(rDotV, 100.0 * specPower) * specMult;
	
	return specularity;
}

void main() {
	vec4 WorldPos = vec4(Position,1.0) * ObjToWorld;
	vec4 ViewPos = WorldPos * WorldToView;
	
	gl_Position = ViewPos;
	Out_Normal = normalize((vec4(Normal, 0.0) * ObjToWorld).xyz);
	Out_Tex0 = Tex0;
	
	// calculate spec
	vec3 LightDir = normalize(DirFromLight.xyz);
	vec3 EyeDir = normalize(EyePosition);
	Out_DirEye.x = GetSpecularity(EyeDir, LightDir, Out_Normal, SpecParam.x, SpecParam.y);
	
	// calculate rim
	Out_DirEye.y = clamp((1.0 - pow( dot(Out_Normal, -EyeDir), RimParam.x)) * RimParam.y, 0.0, RimParam.z);
	
	// calculate direct lighting (with bounce light)
	float lightAmount = dot(Out_Normal, LightDir);
	Out_DirEye.z = lightAmount < 0.0 ? lightAmount * -LightColor.a : lightAmount;
	
	Out_Lighting = Out_DirEye.z * LightColor + AmbientColor;
	
	Out_LightTex0 = (WorldPos * LightViewProjection).xy * 0.5 + 0.5;
}